@@ -690,10 +690,11 @@ class QueryCompiler(object):
             p = []
         elif isinstance(expr, SelectQuery):
             max_alias = self._max_alias(alias_map)
+            alias_copy = alias_map and alias_map.copy() or None
             clone = expr.clone()
             if not expr._explicit_selection:
                 clone._select = (clone.model_class._meta.primary_key,)
-            subselect, p = self.generate_select(clone, max_alias, alias_map)
+            subselect, p = self.generate_select(clone, max_alias, alias_copy)
             s = '(%s)' % subselect
         elif isinstance(expr, (list, tuple)):
             exprs = []
@@ -965,27 +966,12 @@ class QueryResultWrapper(object):
     """
     Provides an iterator over the results of a raw Query, additionally doing
     two things:
-    - converts rows from the database into model instances
+    - converts rows from the database into python representations
     - ensures that multiple iterations do not result in multiple queries
     """
     def __init__(self, model, cursor, meta=None):
         self.model = model
         self.cursor = cursor
-        self.naive = not meta
-
-        if self.naive:
-            cols = []
-            non_cols = []
-            for i in range(len(self.cursor.description)):
-                col = self.cursor.description[i][0]
-                if col in model._meta.columns:
-                    cols.append((i, model._meta.columns[col]))
-                else:
-                    non_cols.append((i, col))
-            self._cols = cols
-            self._non_cols = non_cols
-        else:
-            self.column_meta, self.join_meta = meta
 
         self.__ct = 0
         self.__idx = 0
@@ -993,14 +979,94 @@ class QueryResultWrapper(object):
         self._result_cache = []
         self._populated = False
 
-    def simple_iter(self, row):
+    def __iter__(self):
+        self.__idx = 0
+
+        if not self._populated:
+            return self
+        else:
+            return iter(self._result_cache)
+
+    def process_row(self, row):
+        return row
+
+    def iterate(self):
+        row = self.cursor.fetchone()
+        if not row:
+            self._populated = True
+            raise StopIteration
+        return self.process_row(row)
+
+    def iterator(self):
+        while True:
+            yield self.iterate()
+
+    def next(self):
+        if self.__idx < self.__ct:
+            inst = self._result_cache[self.__idx]
+            self.__idx += 1
+            return inst
+
+        obj = self.iterate()
+        self._result_cache.append(obj)
+        self.__ct += 1
+        self.__idx += 1
+        return obj
+
+    def fill_cache(self, n=None):
+        n = n or float('Inf')
+        self.__idx = self.__ct
+        while not self._populated and (n > self.__ct):
+            try:
+                self.next()
+            except StopIteration:
+                break
+
+class NaiveQueryResultWrapper(QueryResultWrapper):
+    def __init__(self, model, cursor, meta=None):
+        super(NaiveQueryResultWrapper, self).__init__(model, cursor, meta)
+
+        cols = []
+        non_cols = []
+        for i in range(len(self.cursor.description)):
+            col = self.cursor.description[i][0]
+            if col in model._meta.columns:
+                field_obj = model._meta.columns[col]
+                cols.append((i, field_obj.name, field_obj.python_value))
+            else:
+                non_cols.append((i, col))
+        self._cols = cols
+        self._non_cols = non_cols
+
+    def process_row(self, row):
         instance = self.model()
-        for i, f in self._cols:
-            setattr(instance, f.name, f.python_value(row[i]))
+        for i, fname, pv in self._cols:
+            setattr(instance, fname, pv(row[i]))
         for i, f in self._non_cols:
             setattr(instance, f, row[i])
+        instance.prepared()
         return instance
 
+class DictQueryResultWrapper(NaiveQueryResultWrapper):
+    def process_row(self, row):
+        res = {}
+        for i, fname, pv in self._cols:
+            res[fname] = pv(row[i])
+        for i, f in self._non_cols:
+            res[f] = row[i]
+        return res
+
+class ModelQueryResultWrapper(QueryResultWrapper):
+    def __init__(self, model, cursor, meta=None):
+        super(ModelQueryResultWrapper, self).__init__(model, cursor, meta)
+        self.column_meta, self.join_meta = meta
+
+    def process_row(self, row):
+        collected = self.construct_instance(row)
+        instances = self.follow_joins(collected)
+        map(lambda i: i.prepared(), instances)
+        return instances[0]
+
     def construct_instance(self, row):
         # we have columns, models, and a graph of joins to reconstruct
         collected_models = {}
@@ -1031,6 +1097,7 @@ class QueryResultWrapper(object):
     def follow_joins(self, collected):
         joins = self.join_meta
         stack = [self.model]
+        prepared = [collected[self.model]]
         while stack:
             current = stack.pop()
             if current not in joins:
@@ -1053,54 +1120,9 @@ class QueryResultWrapper(object):
 
                     setattr(inst, fk_field.name, joined_inst)
                     stack.append(join.model_class)
+                    prepared.append(joined_inst)
 
-        return collected[self.model]
-
-    def __iter__(self):
-        self.__idx = 0
-
-        if not self._populated:
-            return self
-        else:
-            return iter(self._result_cache)
-
-    def iterate(self):
-        row = self.cursor.fetchone()
-        if not row:
-            self._populated = True
-            raise StopIteration
-
-        if self.naive:
-            return self.simple_iter(row)
-        else:
-            collected = self.construct_instance(row)
-            return self.follow_joins(collected)
-
-    def iterator(self):
-        while 1:
-            yield self.iterate()
-
-    def next(self):
-        if self.__idx < self.__ct:
-            inst = self._result_cache[self.__idx]
-            self.__idx += 1
-            return inst
-
-        instance = self.iterate()
-        instance.prepared() # <-- model prepared hook
-        self._result_cache.append(instance)
-        self.__ct += 1
-        self.__idx += 1
-        return instance
-
-    def fill_cache(self, n=None):
-        n = n or float('Inf')
-        self.__idx = self.__ct
-        while not self._populated and (n > self.__ct):
-            try:
-                self.next()
-            except StopIteration:
-                break
+        return prepared
 
 Join = namedtuple('Join', ('model_class', 'join_type', 'on'))
 
@@ -1247,21 +1269,40 @@ class RawQuery(Query):
         self._sql = query
         self._params = list(params)
         self._qr = None
+        self._tuples = False
+        self._dicts = False
         super(RawQuery, self).__init__(model)
 
     def clone(self):
-        return RawQuery(self.model_class, self._sql, *self._params)
+        query = RawQuery(self.model_class, self._sql, *self._params)
+        query._tuples = self._tuples
+        query._dicts = self._dicts
+        return query
 
     join = not_allowed('joining')
     where = not_allowed('where')
     switch = not_allowed('switch')
 
+    @returns_clone
+    def tuples(self, tuples=True):
+        self._tuples = tuples
+
+    @returns_clone
+    def dicts(self, dicts=True):
+        self._dicts = dicts
+
     def sql(self):
         return self._sql, self._params
 
     def execute(self):
         if self._qr is None:
-            self._qr = QueryResultWrapper(self.model_class, self._execute(), None)
+            if self._tuples:
+                ResultWrapper = QueryResultWrapper
+            elif self._dicts:
+                ResultWrapper = DictQueryResultWrapper
+            else:
+                ResultWrapper = NaiveQueryResultWrapper
+            self._qr = ResultWrapper(self.model_class, self._execute(), None)
         return self._qr
 
     def __iter__(self):
@@ -1282,6 +1323,8 @@ class SelectQuery(Query):
         self._distinct = False
         self._for_update = False
         self._naive = False
+        self._tuples = False
+        self._dicts = False
         self._alias = None
         self._qr = None
 
@@ -1300,6 +1343,8 @@ class SelectQuery(Query):
         query._distinct = self._distinct
         query._for_update = self._for_update
         query._naive = self._naive
+        query._tuples = self._tuples
+        query._dicts = self._dicts
         query._alias = self._alias
         return query
 
@@ -1359,6 +1404,14 @@ class SelectQuery(Query):
     def naive(self, naive=True):
         self._naive = naive
 
+    @returns_clone
+    def tuples(self, tuples=True):
+        self._tuples = tuples
+
+    @returns_clone
+    def dicts(self, dicts=True):
+        self._dicts = dicts
+
     @returns_clone
     def alias(self, alias=None):
         self._alias = alias
@@ -1430,11 +1483,17 @@ class SelectQuery(Query):
 
     def execute(self):
         if self._dirty or not self._qr:
-            if self._naive or not self._joins or self.verify_naive():
-                query_meta = None
+            query_meta = None
+            if self._tuples:
+                ResultWrapper = QueryResultWrapper
+            elif self._dicts:
+                ResultWrapper = DictQueryResultWrapper
+            elif self._naive or not self._joins or self.verify_naive():
+                ResultWrapper = NaiveQueryResultWrapper
             else:
                 query_meta = [self._select, self._joins]
-            self._qr = QueryResultWrapper(self.model_class, self._execute(), query_meta)
+                ResultWrapper = ModelQueryResultWrapper
+            self._qr = ResultWrapper(self.model_class, self._execute(), query_meta)
             self._dirty = False
             return self._qr
         else:
@@ -1932,6 +1991,8 @@ class BaseModel(type):
         if meta:
             meta_options.update((k, v) for k, v in meta.__dict__.items() if not k.startswith('_'))
 
+        orig_primary_key = None
+
         # inherit any field descriptors by deep copying the underlying field obj
         # into the attrs of the new model, additionally see if the bases define
         # inheritable model options and swipe them
@@ -1948,6 +2009,8 @@ class BaseModel(type):
                 if isinstance(v, FieldDescriptor) and k not in attrs:
                     if not v.field.primary_key:
                         attrs[k] = deepcopy(v.field)
+                    elif not orig_primary_key:
+                        orig_primary_key = deepcopy(v.field)
 
         # initialize the new class and set the magic attributes
         cls = super(BaseModel, cls).__new__(cls, name, bases, attrs)
@@ -1964,12 +2027,17 @@ class BaseModel(type):
                 if attr.primary_key:
                     primary_key = attr
 
-        if not primary_key:
-            primary_key = PrimaryKeyField(primary_key=True)
-            primary_key.add_to_class(cls, 'id')
+        if primary_key is None:
+            if orig_primary_key:
+                primary_key = orig_primary_key
+                name = primary_key.name
+            else:
+                primary_key = PrimaryKeyField(primary_key=True)
+                name = 'id'
+            primary_key.add_to_class(cls, name)
 
         cls._meta.primary_key = primary_key
-        cls._meta.auto_increment = isinstance(primary_key, PrimaryKeyField) or primary_key.sequence
+        cls._meta.auto_increment = isinstance(primary_key, PrimaryKeyField) or bool(primary_key.sequence)
         if not cls._meta.db_table:
             cls._meta.db_table = re.sub('[^\w]+', '_', cls.__name__.lower())
 
@@ -2188,8 +2256,8 @@ def prefetch_add_subquery(sq, subqueries):
             subquery = subquery.select()
         subquery_model = subquery.model_class
         fkf = None
-        for j in range(i + 1):
-            last_query = fixed_queries[i - j - 1][0]
+        for j in range(i, -1, -1):
+            last_query = fixed_queries[j][0]
             fkf = subquery_model._meta.rel_for_model(last_query.model_class)
             if fkf:
                 break
